<?php
namespace nur\sery\wip\schema\types;

use nur\sery\wip\schema\_scalar\ScalarResult;
use nur\sery\wip\schema\_scalar\ScalarSchema;
use nur\sery\wip\schema\Result;
use nur\sery\wip\schema\Schema;

class trawstring extends _tsimple {
  const TRIM = false;

  static function ensure_string(&$string): void {
    if (!is_string($string)) $string = strval($string);
    if (static::TRIM) $string = trim($string);
  }

  static function ensure_nstring(&$string): void {
    if ($string !== null) self::ensure_string($string);
  }

  function __construct(?array $params=null) {
    parent::__construct($params);
    $this->trim = boolval($this->params["trim"] ?? static::TRIM);
  }

  protected bool $trim;

  function isNull($value): bool {
    return $value === null;
  }

  protected function check_normalized(string $value, bool $trim): bool {
    if ($trim && $value !== "") {
      if (preg_match('/^\s+/', $value)) {
        return false;
      } elseif (preg_match('/\s+$/', $value)) {
        return false;
      }
    }
    return true;
  }

  function isValid($value, ?bool &$normalized=null): bool {
    if (is_string($value)) {
      $normalized = self::check_normalized($value, $this->trim);
      return true;
    }
    return is_scalar($value);
  }
  
  /**
   * @var ScalarResult $result
   * @var ScalarSchema $schema
   */
  function verifix(&$value, Result &$result, Schema $schema): bool {
    if (is_string($value)) {
      $normalized = false;
      if ($this->trim) {
        $orig = $value;
        $value = trim($value);
        $normalized = $value !== $orig;
      }
      $result->setNormalized();
      return $normalized;
    } elseif (is_scalar($value)) {
      $value = strval($value);
      if ($this->trim) $value = trim($value);
      $result->setNormalized();
      return true;
    } else {
      $result->setInvalid($value, $schema);
      return false;
    }
  }

  function format($value, $format=null): string {
    return strval($value);
  }
}
